@@ -1,6 +1,6 @@
 import cro_validate.classes.definition_classes as Definitions
 import cro_validate.classes.util_classes as Utils
-from cro_validate.enum import DataType
+from cro_validate.enum import DataType, ValidationMode
 
 
 def get(definition_or_name, version=Utils.Empty):
@@ -42,18 +42,53 @@ def from_json_dict(root):
 	return Definitions.Index.from_json_dict(root)
 
 
-def add_version(version):
-	return Definitions.Index.add_version(version)
+def add_version(version, display_name=None):
+	return Definitions.Index.add_version(version, display_name)
+
+
+def get_version_display_name(version):
+	return Definitions.Index.get_version_display_name(version)
+
+
+def get_version_by_display_name(display_name):
+	return Definitions.Index.get_version_by_display_name(display_name)
 
 
 def list_versions():
 	return Definitions.Index.list_versions()
 
+def compare_versions(a, b):
+	return Definitions.Index.compare_versions(a, b)
+
+
+def list_definition_versions(definition_or_name, include_identical=True):
+	return Definitions.Index.list_definition_versions(definition_or_name, include_identical)
+
+
+def definition_has_version(definition_or_name, version):
+	return Definitions.Index.definition_has_version(definition_or_name, version)
+
+
+def get_definition_base_version(definition_or_name):
+	return Definitions.Index.get_definition_base_version(definition_or_name)
+
+
+def get_definition_final_version(definition_or_name):
+	return Definitions.Index.get_definition_final_version(definition_or_name)
+
+
+def get_definition_previous_version(definition_or_name, version):
+	return Definitions.Index.get_definition_previous_version(definition_or_name, version)
+
 
 def get_latest_version():
 	return Definitions.Index.get_latest_version()
 
 
+def get_first_version():
+	return Definitions.Index.get_first_version()
+
+
 def version_exists(version):
 	return Definitions.Index.version_exists(version)
 
@@ -117,13 +152,6 @@ def list_dependent_definitions(definition_name):
 	return results
 
 
-def list_fields(name_or_definition):
-	definition = name_or_definition
-	if not isinstance(definition, Definitions.Definition):
-		definition = get(str(name_or_definition))
-	return definition.list_fields()
-
-
 def validate_input(
 			definition_or_name,
 			value,
@@ -132,13 +160,35 @@ def validate_input(
 			version=Utils.Empty,
 			**rules_kw
 		):
-	results = Definitions.Index.validate_input(
+	results = Definitions.Index.validate(
 			None,
 			field_fqn,
 			field_name,
 			definition_or_name,
 			value,
 			version,
+			ValidationMode.Input,
+			**rules_kw
+		)
+	return results
+
+
+def validate_output(
+			definition_or_name,
+			value,
+			field_fqn=None,
+			field_name=None,
+			version=Utils.Empty,
+			**rules_kw
+		):
+	results = Definitions.Index.validate(
+			None,
+			field_fqn,
+			field_name,
+			definition_or_name,
+			value,
+			version,
+			ValidationMode.Output,
 			**rules_kw
 		)
 	return results
