@@ -1,7 +1,7 @@
 import importlib, collections
 
 from enum import Enum
-from cro_validate.enum import DataType, VersionMutationType
+from cro_validate.enum import DataType, VersionMutationType, ValidationMode
 from cro_validate.classes.configuration_classes import Config
 import cro_validate.classes.parameter_classes as Parameters
 import cro_validate.classes.schema_classes as Schemas
@@ -59,9 +59,14 @@ class Definition:
 	deprecated = False
 	internal = False
 	rules = []
+	base_version = 'base'
+	final_version = None
+	version_conversion = None
+	mutations = None
 	meta = DefaultDefinitionMeta()
 
 	def to_json_dict(self):
+		# TODO: Refresh (broken) or delete this functionality.
 		def _is_default(name, value):
 			class_value = getattr(self.__class__, name)
 			if class_value == value:
@@ -121,8 +126,11 @@ class Definition:
 				deprecated,
 				internal,
 				rules,
+				base_version,
+				final_version,
 				version,
 				version_conversion,
+				mutations,
 				meta
 			):
 		def _set_member(name, value):
@@ -140,8 +148,11 @@ class Definition:
 		_set_member('deprecated', deprecated)
 		_set_member('internal', internal)
 		_set_member('rules', rules)
-		self.version = version  # ephemeral (managed by Index)
-		self.version_conversion = version_conversion
+		_set_member('base_version', base_version)
+		_set_member('final_version', final_version)
+		self.version = version  # mandatory
+		_set_member('version_conversion', version_conversion)
+		_set_member('mutations', mutations)
 		_set_member('meta', meta)
 		# Name
 		######
@@ -173,11 +184,14 @@ class Definition:
 		# Validator
 		###########
 		if self.is_object():
-			self.validator = self._get_obj_validator()
+			self.input_validator = self._get_obj_validator(ValidationMode.Input)
+			self.output_validator = self._get_obj_validator(ValidationMode.Output)
 		elif self.is_array():
-			self.validator = self._validate_array
+			self.input_validator = self._validate_array
+			self.output_validator = self._validate_array
 		else:
-			self.validator = self._assign_value
+			self.input_validator = self._assign_value
+			self.output_validator = self._assign_value
 		# Examples
 		##########
 		if not self.examples:
@@ -189,8 +203,8 @@ class Definition:
 		######
 		self.meta.initialize(self)
 
-	def _get_obj_validator(self):
-		model_validator = Schemas.ModelValidator(self.data_format)
+	def _get_obj_validator(self, mode):
+		model_validator = Schemas.ModelValidator(self.data_format, self.get_version(), mode)
 		validator = Schemas.Validator(self.name, model_validator)
 		return validator
 
@@ -201,6 +215,7 @@ class Definition:
 				field_name,
 				definition,
 				value,
+				mode,
 				**rules_kw):
 		if isinstance(value, list):
 			pass
@@ -211,13 +226,14 @@ class Definition:
 		items = []
 		i = 0
 		for entry in value:
-			item = Index.validate_input(
+			item = Index.validate(
 					validated=None,
 					field_fqn=field_fqn + '[' + str(i) + ']',
 					field_name=field_name,
 					definition_or_name=self.data_format,
 					value=entry,
 					version=self.get_version(),
+					mode=mode,
 					**rules_kw
 				)
 			items.append(item[field_name])
@@ -231,6 +247,7 @@ class Definition:
 				field_name,
 				definition,
 				value,
+				mode,
 				**rules_kw):
 		results[field_name] = value
 
@@ -241,21 +258,25 @@ class Definition:
 				field_name,
 				definition,
 				value,
+				mode,
 				**rules_kw
 			):
 		'''
 		The validate func
 		'''
+		validator = self.input_validator
+		if mode is ValidationMode.Output:
+			validator = self.output_validator
 		try:
 			results = Parameters.Index.ensure(results)
-			if not self.validator:
+			if not validator:
 				raise Config.exception_factory.create_internal_error(self.name, "Missing validator.")
 			if field_name is None:
 				field_name = self.name
 			if field_fqn is None:
 				field_fqn = field_name
 				if self.data_type == DataType.Object:
-					field_fqn = self.validator.model_validator.name
+					field_fqn = validator.model_validator.name
 			if value is None:
 				if self.nullable is True:
 					results[field_name] = None
@@ -277,12 +298,13 @@ class Definition:
 					self,
 					**rules_kw
 				)
-			self.validator(
+			validator(
 					results,
 					field_fqn,
 					field_name,
 					self,
 					results[field_name],
+					mode,
 					**rules_kw
 				)
 			for rule in self.rules:
@@ -299,9 +321,11 @@ class Definition:
 			return False
 		return True
 
-	def get_default_value(self, name):
+	def get_default_value(self, field_name=None):
 		if not self.has_default_value():
-			raise Config.exception_factory.create_internal_error(self.name, 'No default value configured')
+			if field_name is None:
+				field_name = self.name
+			raise Config.exception_factory.create_internal_error(field_name, 'No default value configured')
 		return self.default_value
 
 	def get_name(self):
@@ -313,6 +337,15 @@ class Definition:
 	def get_version_conversion(self):
 		return self.version_conversion
 
+	def get_base_version(self):
+		return self.base_version
+
+	def get_final_version(self):
+		return self.final_version
+
+	def get_mutations(self):
+		return self.mutations
+
 	def get_data_type(self):
 		return self.data_type
 
@@ -355,6 +388,14 @@ class Definition:
 	def is_nullable(self):
 		return self.nullable
 
+	def is_deprecated(self):
+		return self.deprecated
+
+	def is_obsolete(self):
+		if self.final_version is not None:
+			return True
+		return False
+
 	def list_fields(self):
 		if self.data_type is not DataType.Object:
 			return {}
@@ -370,7 +411,7 @@ class Definition:
 					continue
 				fields[name] = getattr(model, name)
 				if fields[name] is None:
-					fields[name] = Schemas.Field(input_name=name, definition_name=Index.get(name))
+					fields[name] = Schemas.Field(input_name=name, definition_name=name)
 				elif fields[name].definition_name is None:
 					fields[name].definition_name = name
 		for k in fields:
@@ -466,6 +507,7 @@ class DefinitionJsonDeserializer:
 				init_action_class_name = entry['type']
 				init_action = instantiate(init_action_class_name, **init_action_kw)
 				field_init_actions.append(init_action)
+		kw['field_init_actions'] = field_init_actions
 		if 'dependency_resolver' in obj:
 			resolver_class_name = obj['dependency_resolver']['type']
 			resolver_kw = {}
@@ -478,26 +520,16 @@ class DefinitionJsonDeserializer:
 				if not Index.exists(inherited_definition_name):
 					dependent_definition_profile = self.deserialize(inherited_definition_name)
 					Index.register_definition(**dependent_definition_profile)
-				parent_definition = Index.get(inherited_definition_name)
-				inherited_fields = parent_definition.list_fields()
-				for field_name in inherited_fields:
-					model[field_name] = inherited_fields[field_name]
+				#parent_definition = Index.get(inherited_definition_name)
+				#inherited_fields = parent_definition.list_fields()
+				#for field_name in inherited_fields:
+				#	model[field_name] = inherited_fields[field_name]
+			kw['inherits'] = obj['inherits']
 		if 'model' in obj:
 			for field_name in obj['model']:
 				field = self._deserialize_schema_field(field_name, obj['model'][field_name])
 				model[field_name] = field
-		initialized_model = {}
-		for action in field_init_actions:
-			action.validate_schema_model(model)
-		for field_name in model:
-			field = model[field_name]
-			for action in field_init_actions:
-				field = action(field_name, field)
-				if field is None:
-					break
-			if field is not None:
-				initialized_model[field_name] = field
-		kw['model'] = initialized_model
+		kw['model'] = model
 		schema = Schemas.Schema(**kw)
 		return schema
 
@@ -511,16 +543,21 @@ class DefinitionJsonDeserializer:
 		for mutation in obj['mutations']:
 			action = mutation['action']
 			action = VersionMutationType[action]
-			if action == VersionMutationType.AddField:
-				mutations.append(Utils.VersionMutation(action, mutation['config'], None))
-			elif action == VersionMutationType.RemoveField:
-				mutations.append(Utils.VersionMutation(action, mutation['config'], None))
-			elif action == VersionMutationType.MutateField:
-				mutations.append(Utils.VersionMutation(action, mutation['config'], None))
-			else:
-				raise Exception('Not Implemented')
-			if 'definition_name' in mutation['config']:
-				definition_name = mutation['config']['definition_name']
+			description = None
+			if 'description' in mutation:
+				description = mutation['description']
+			config = mutation['config']
+			supported_mutations = {
+					VersionMutationType.AddField,
+					VersionMutationType.RemoveField,
+					VersionMutationType.MutateField,
+					VersionMutationType.RenameField
+				}
+			if action not in supported_mutations:
+				raise Exception('Version Mutation Not Implemented: {0}'.format(action))
+			mutations.append(Utils.VersionMutation(action, config, description))
+			if 'definition_name' in config:
+				definition_name = config['definition_name']
 				if not Index.exists(definition_name):
 					dependent_definition_profile = self.deserialize(definition_name)
 					Index.register_definition(**dependent_definition_profile)
@@ -585,6 +622,8 @@ class DefinitionJsonDeserializer:
 		##########
 		if 'base_version' in obj:
 			kw['base_version'] = obj['base_version']
+		if 'final_version' in obj:
+			kw['final_version'] = obj['final_version']
 		if 'base_version_conversion' in obj:
 			config = obj['base_version_conversion']['config']
 			type_fqn = obj['base_version_conversion']['type']
@@ -592,6 +631,11 @@ class DefinitionJsonDeserializer:
 		if 'versions' in obj:
 			kw['versions'] = {}
 			for version in obj['versions']:
+				if Index.version_exists(version) != True:
+					raise Config.exception_factory.create_internal_error(
+							name,
+							'Unkown version found during deserialization: {0}.'.format(version)
+						)
 				kw['versions'][version] = self._deserialize_version(obj['versions'][version])
 		# Simple
 		########
@@ -612,16 +656,58 @@ class Index:
 	_data_format_rule_exceptions = set()
 	_versions = ['base']
 	_versions_idx = {'base': 0}
+	_versions_display_names = {'base': None}
 
-	def add_version(version):
+	def _resolve_definition_name(definition_or_name, error_on_missing=True):
+		definition_name = definition_or_name
+		if isinstance(definition_or_name, Definition) is True:
+			definition_name = definition_or_name.get_name()
+		resolved = Config.definition_name_resolver.resolve(Index._idx, definition_name)
+		if resolved is None and error_on_missing is True:
+			raise Config.exception_factory.create_internal_error(
+					definition_name,
+					'Definition name resolution failed (Unknown definition name).'
+				)
+		return resolved
+
+	def add_version(version, display_name):
 		if version in Index._versions_idx:
-			return
+			raise Config.exception_factory.create_internal_error(
+					version,
+					'Version already exists: {0}'.format(version)
+				)
+		if display_name is not None:
+			if Index.get_version_by_display_name(display_name) is not None:
+				raise Config.exception_factory.create_internal_error(
+						version,
+						'Version with display name already exists: {0}'.format(display_name)
+					)
 		latest_version = Index.get_latest_version()
 		for k in Index._idx:
+			final_definition_version = Index.get_definition_final_version()
+			if Index.compare_versions(final_definition_version, latest_version) < 0:
+				continue
 			latest_def = Index.get(k, version=latest_version)
 			Index._idx[k][version] = latest_def
 		Index._versions_idx[version] = len(Index._versions)
 		Index._versions.append(version)
+		Index._versions_display_names[version] = display_name
+
+	def get_version_display_name(version):
+		if version not in Index._versions_idx:
+			raise Config.exception_factory.create_internal_error(
+					version,
+					'No such version: {0}'.format(version)
+				)
+		result = Index._versions_display_names[version]
+		return result
+
+	def get_version_by_display_name(display_name):
+		for version in reversed(Index.list_versions()):
+			n = Index._versions_display_names[version]
+			if n == display_name:
+				return version
+		return None
 
 	def list_versions():
 		result = []
@@ -631,40 +717,113 @@ class Index:
 	def get_latest_version():
 		return Index._versions[-1]
 
+	def get_first_version():
+		return Index._versions[1]
+
 	def version_exists(version):
 		if version in Index._versions_idx:
 			return True
 		return False
 
+	def list_definition_versions(definition_or_name, include_identical):
+		base_definition = Index.get_definition_base_version(definition_or_name)
+		versions = Index._idx[base_definition.get_name()]
+		if include_identical is True:
+			result = [versions[v].get_version() for v in Index._versions if v in versions]
+			return result
+		unique = set()
+		unique.add(base_definition.get_version())
+		for version in versions:
+			definition = versions[version]
+			mutations = definition.get_mutations()
+			if len(mutations) < 1:
+				continue
+			unique.add(definition.get_version())
+		result = [versions[v].get_version() for v in Index._versions if v in unique]
+		return result
+
+	def get_definition_base_version(definition_or_name):
+		resolved = Index._resolve_definition_name(definition_or_name)
+		for version in Index._versions:
+			if version in Index._idx[resolved]:
+				return Index._idx[resolved][version]
+		raise Config.exception_factory.create_internal_error(
+				resolved,
+				'Definition version resolution failed (base version not found).'
+			)
+
+	def get_definition_final_version(definition_or_name):
+		resolved = Index._resolve_definition_name(definition_or_name)
+		for version in reversed(Index._versions):
+			if version in Index._idx[resolved]:
+				return Index._idx[resolved][version]
+		raise Config.exception_factory.create_internal_error(
+				resolved,
+				'Definition version resolution failed (final version not found).'
+			)
+
+	def get_definition_previous_version(definition_or_name, version):
+		versions = Index.list_definition_versions(definition_or_name, include_identical=True)
+		i = len(versions) - 1
+		while i > 1:
+			if versions[i] == version:
+				previous_version = versions[i-1]
+				result = Index.get(definition_or_name, previous_version)
+				return result
+			i = i - 1
+		resolved = Index._resolve_definition_name(definition_or_name)
+		raise Config.exception_factory.create_internal_error(
+				resolved,
+				'No previous version found (version={0}).'.format(version)
+			)
+
 	def get(definition_or_name, version=Utils.Empty):
-		definition_name = definition_or_name
-		if isinstance(definition_name, Definition) is True:
-			definition_name = definition_or_name.get_name()
+		definition_name = Index._resolve_definition_name(definition_or_name)
 		if Utils.Empty.isempty(version) is not True:
 			if version not in Index._versions:
-				raise Config.exception_factory.create_internal_error(definition_name, 'Unknown/unregistered version requested: {0}'.format(version))
-		resolved = Config.definition_name_resolver.resolve(Index._idx, definition_name)
-		if resolved is None:
-			raise Config.exception_factory.create_internal_error(definition_name, 'Definition name resolution failed (Unknown definition name).')
-		versions = Index._idx[resolved]
+				raise Config.exception_factory.create_internal_error(
+						definition_name,
+						'Unknown/unregistered version requested: {0}'.format(version)
+					)
+		versions = Index._idx[definition_name]
 		# Latest
 		########
 		if Utils.Empty.isempty(version) is True:
 			latest_version = Index.get_latest_version()
-			return versions[latest_version]
+			final_definition = Index.get_definition_final_version(definition_name)
+			if Index.compare_versions(latest_version, final_definition.get_version()) == 1:
+				raise Config.exception_factory.create_internal_error(
+						definition_name,
+						'Version requested after final version: {0}'.format(version)
+					)
+			return final_definition
 		# Base
 		######
 		if version == 'base':
-			for next_version in Index._versions:
-				if next_version in versions:
-					return versions[next_version]
-			raise Config.exception_factory.create_internal_error(definition_name, 'Base version not found: {0}'.format(version))
+			base_version_definition = Index.get_definition_base_version(definition_name)
+			return base_version_definition
 		# Specified
 		###########
+		base_definition = Index.get_definition_base_version(definition_name)
+		final_definition = Index.get_definition_final_version(definition_name)
+		if Index.compare_versions(version, base_definition.get_version()) == -1:
+			raise Config.exception_factory.create_internal_error(
+					definition_name,
+					'Version requested falls before base version: {0}'.format(version)
+				)
+		if Index.compare_versions(version, final_definition.get_version()) == 1:
+			raise Config.exception_factory.create_internal_error(
+				definition_name,
+				'Version requested falls after final version: {0}'.format(version)
+			)
+
 		return versions[version]
 
-	def exists(name):
-		resolved = Config.definition_name_resolver.resolve(Index._idx, name)
+	def exists(definition_name):
+		resolved = Index._resolve_definition_name(
+				definition_name,
+				error_on_missing=False
+			)
 		if resolved is None:
 			return False
 		return True
@@ -690,7 +849,38 @@ class Index:
 			profile = deserializer.deserialize(k)
 			Index.register_definition(**profile)
 
-	def mutate_model(definition_name, base_model, versions, target_version):
+	def compare_versions(a, b):
+		if a not in Index._versions_idx:
+			raise Config.exception_factory.create_internal_error(
+					'compare_versions',
+					'Source version not found: {0}.'.format(a)
+				)
+		a_idx = Index._versions_idx[a]
+		if b not in Index._versions_idx:
+			raise Config.exception_factory.create_internal_error(
+					'compare_versions',
+					'Target version not found: {0}.'.format(b)
+				)
+		b_idx = Index._versions_idx[b]
+		if a_idx < b_idx:
+			return -1
+		if a_idx == b_idx:
+			return 0
+		return 1
+
+	def definition_has_version(definition_or_name, version):
+		definition_name = definition_or_name
+		if isinstance(definition_or_name, Definition) is True:
+			definition_name = definition_or_name.get_name()
+		base_definition = Index.get_definition_base_version(definition_name)
+		final_definition = Index.get_definition_final_version(definition_name)
+		if Index.compare_versions(base_definition.get_version(), version) == 1:
+			return False
+		if Index.compare_versions(final_definition.get_version(), version) == -1:
+			return False
+		return True
+
+	def _mutate_model(definition_name, base_model, versions, target_version):
 		# Init Mutation
 		###############
 		mutated_model = {}
@@ -699,7 +889,7 @@ class Index:
 				if k.startswith('_'):
 					continue
 				if isinstance(base_model[k], Schemas.Field):
-					mutated_model[k] = base_model[k]
+					mutated_model[k] = base_model[k].copy()
 				elif isinstance(base_model[k], type(None)):
 					mutated_model[k] = Schemas.Field(input_name=k)
 		else:
@@ -708,7 +898,7 @@ class Index:
 					continue
 				v = getattr(base_model, k)
 				if isinstance(v, Schemas.Field):
-					mutated_model[k] = v
+					mutated_model[k] = v.copy()
 				elif isinstance(v, type(None)):
 					mutated_model[k] = Schemas.Field(input_name=k)
 		# Mutate
@@ -722,11 +912,11 @@ class Index:
 			for mutation in mutations:
 				if mutation.action == VersionMutationType.AddField:
 					field = Schemas.Field(**mutation.config)
-					field_name = field.output_name
+					field_name = field.input_name
 					mutated_model[field_name] = field
 				elif mutation.action == VersionMutationType.RemoveField:
 					field = Schemas.Field(**mutation.config)
-					del mutated_model[field.output_name]
+					del mutated_model[field.input_name]
 				elif mutation.action == VersionMutationType.MutateField:
 					old_field = mutated_model[mutation.config['field_name']]
 					config = {
@@ -735,11 +925,15 @@ class Index:
 							if k != 'field_name'
 						}
 					new_field = Schemas.Field(
-							input_name=old_field.input_name,
-							output_name=old_field.output_name
+							input_name=old_field.input_name
 						)
 					new_field.mutate(**config)
-					mutated_model[new_field.output_name] = new_field
+					mutated_model[mutation.config['field_name']] = new_field
+				elif mutation.action == VersionMutationType.RenameField:
+					old_field_name = mutation.config['field_name']
+					old_field = mutated_model[old_field_name]
+					del mutated_model[old_field_name]
+					mutated_model[mutation.config['new_field_name']] = old_field
 			if version == target_version:
 				found = True
 				break
@@ -750,6 +944,34 @@ class Index:
 				)
 		return mutated_model
 
+	def _add_inherited_fields(inherited_definition_name, version, model):
+		parent_definition = Index.get(inherited_definition_name, version)
+		inherited_fields = parent_definition.list_fields()
+		for field_name in inherited_fields:
+			model[field_name] = inherited_fields[field_name]
+
+	def _extend_model(data_format, version):
+		model = {}
+		for inherited_definition_name in data_format.inherits:
+			Index._add_inherited_fields(inherited_definition_name, version, model)
+		for field_name in data_format.model:
+			field = data_format.model[field_name]
+			for action in data_format.field_init_actions:
+				field = action(field_name, field)
+				if field is None:
+					break
+			if field is not None:
+				model[field_name] = field
+		for action in data_format.field_init_actions:
+			action.validate_schema_model(model)
+		data_format.model = model
+
+	def _extend_mutations(mutations, data_format, version):
+		# TODO: Should traverse inheritance heirarchy, not just parent
+		for inherited_definition_name in data_format.inherits:
+			inherited_definition = Index.get(inherited_definition_name, version)
+			mutations.extend(inherited_definition.get_mutations())
+
 	def register_definition(
 				name,
 				aliases,
@@ -765,12 +987,13 @@ class Index:
 				meta,
 				base_version=Utils.Empty,
 				base_version_conversion=Utils.Empty,
+				final_version=Utils.Empty,
 				versions=Utils.Empty
 			):
 		if Utils.Empty.isempty(base_version) is True:
 			base_version = 'base'
-		if Utils.Empty.isempty(base_version_conversion) is True:
-			base_version_conversion = None
+		if data_type is DataType.Object:
+			Index._extend_model(data_format, base_version)
 		base_definition = Definition(
 				name=name,
 				aliases=aliases,
@@ -783,8 +1006,11 @@ class Index:
 				deprecated=deprecated,
 				internal=internal,
 				rules=rules,
+				base_version=base_version,
+				final_version=final_version,
 				version=base_version,
 				version_conversion=base_version_conversion,
+				mutations={},
 				meta=meta
 			)
 		name = base_definition.get_name()
@@ -796,10 +1022,14 @@ class Index:
 		definition_versions = {}
 		latest_def_version = base_definition
 		if Utils.Empty.isempty(versions) is True:
-			versions = []
+			versions = {}
 		initial_version_index = Index._versions_idx[base_version]
 		for version in Index._versions[initial_version_index:]:
+			if Utils.Empty.isempty(final_version) is False:
+				if Index.compare_versions(version, final_version) == 1:
+					break
 			if version not in versions:
+				# Index version w/no mutations
 				latest_def_version = Definition(
 						name=latest_def_version.name,
 						aliases=latest_def_version.aliases,
@@ -812,16 +1042,25 @@ class Index:
 						deprecated=latest_def_version.deprecated,
 						internal=latest_def_version.internal,
 						rules=latest_def_version.rules,
+						base_version=latest_def_version.base_version,
+						final_version=latest_def_version.final_version,
 						version=version,
 						version_conversion=latest_def_version.version_conversion,
+						mutations={},
 						meta=latest_def_version.meta
 					)
 				definition_versions[version] = latest_def_version
 				continue
-			mutated_data_format = data_format
+			if Utils.Empty.isempty(final_version) is False:
+				if Index.compare_versions(version, final_version) == 1:
+					break
+			mutated_data_format = latest_def_version.get_data_format().copy()
+			mutations = []
 			if data_type == DataType.Object:
-				mutated_data_format = data_format.copy()
-				mutated_data_format.model = Index.mutate_model(name, mutated_data_format.model, versions, version)
+				Index._extend_model(mutated_data_format, version)
+				mutated_data_format.model = Index._mutate_model(name, mutated_data_format.model, versions, version)
+				Index._extend_mutations(mutations, mutated_data_format, version)
+			mutations.extend(versions[version]['mutations'])
 			version_conversion = None
 			if 'conversion' in versions[version]:
 				version_conversion = versions[version]['conversion']
@@ -837,12 +1076,18 @@ class Index:
 					deprecated=deprecated,
 					internal=internal,
 					rules=rules,
+					base_version=base_version,
+					final_version=final_version,
 					version=version,
 					version_conversion=version_conversion,
+					mutations = mutations,
 					meta=meta
 				)
 			definition_versions[version] = mutated_definition
 			latest_def_version = mutated_definition
+			if Utils.Empty.isempty(final_version) is False:
+				if final_version == version:
+					break
 		names = set()
 		names.add(name)
 		if Utils.Empty.isempty(aliases):
@@ -860,7 +1105,7 @@ class Index:
 				)
 		for entry in names:
 			Index._idx[entry] = definition_versions
-		return Index.get(name)
+		return Index.get(name, version=final_version)
 
 	def register_data_type_rule(data_type, rule, exceptions=[]):
 		if data_type not in Index._data_type_rules:
@@ -907,21 +1152,29 @@ class Index:
 		for rule in Index._data_format_rules[definition.get_data_format()]:
 			results[field_name] = rule.execute(field_fqn, results[field_name], **rules_kw)
 
-	def validate_input(
+	def validate(
 				validated,
 				field_fqn,
 				field_name,
 				definition_or_name,
 				value,
 				version,
+				mode,
 				**rules_kw
 			):
 		definition = Index.get(definition_or_name, version)
 		results = Parameters.Index.ensure(validated)
-		definition.validate(results, field_fqn, field_name, definition, value, **rules_kw)
+		definition.validate(
+				results,
+				field_fqn,
+				field_name,
+				definition,
+				value,
+				mode,
+				**rules_kw
+			)
 		return results
 
-
 	def mutate_input_version(
 				validated,
 				field_fqn,
@@ -941,7 +1194,7 @@ class Index:
 		target_index = Index._versions_idx[target_version]
 		versions = Index._idx[src_definition.get_name()]
 		if skip_validation is False:
-			Index.validate_input(results, field_fqn, field_name, src_definition, value, src_version, **rules_kw)
+			Index.validate(results, field_fqn, field_name, src_definition, value, src_version, ValidationMode.Input, **rules_kw)
 			value = results[src_definition.get_name()]
 		if src_index < target_index:
 			i = src_index
@@ -982,7 +1235,7 @@ class Index:
 		if src_version != target_version:
 			results = {src_definition.get_name(): value}
 			if skip_validation is False:
-				results = Index.validate_input(results, field_fqn, field_name, target_definition, value, target_version, **rules_kw)
+				results = Index.validate(results, field_fqn, field_name, target_definition, value, target_version, ValidationMode.Input, **rules_kw)
 		return results
 
 	def ensure_alias(name, alias):
@@ -995,12 +1248,6 @@ class Index:
 		result.sort()
 		return result
 
-	def list_fields(name):
-		definition = Index.get(name)
-		if definition.data_type == DataType.Object:
-			return definition.validator.list_field_names()
-		return [name]
-
 	def clear():
 		Index._idx = {}
 		Index._data_type_rules = {}
